# -*- coding: UTF-8 -*-
from django.utils.translation import ugettext as _
from django.conf import settings
from django.db.models import Q
from ihx.access.base import definitions, fields, functions, graficos
from ihx.access.core.MODRelatorio import reports as report_relatorio, views as report_views
from app.modules.MODTarifa import models, reports


class vigencia(definitions.baseform):
    # pesquisa
    list_display = ['tempo1', 'tempo2', 'seg', 'ter', 'qua', 'qui', 'sex', 'sab', 'dom']
    list_display_links = list_display
    fieldsets = [
        (_(u'Definições da vigência'), {'fields': ['tempo1', 'tempo2', ]}),
        (_(u'Dias da tarifa'), {'fields': [('seg', 'ter', 'qua', 'qui', 'sex', 'sab', 'dom')]}),
    ]


class tarifa_valor_inline(definitions.basetabularinline):
    model = models.prkvalor
    classes = ('grp-collapse grp-open',)
    inline_classes = ('grp-collapse grp-open',)
    fieldsets = [
        ('Tempo/Valor', {'fields': [('tempo1', 'tempo2', 'valor', 'fracaotempo', 'qtdfracaotempo', 'acumulacobranca', )]}),
                ]
    extra = 0


class tarifa_form(definitions.baseform):
    # pesquisa
    list_display = ['descricao', 'prkvigenciaid', 'tipotarifa', 'tolerancia', 'aposvalor', 'classe']
    ordering = ['descricao', ]
    search_fields = ['descricao', ]
    list_display_links = list_display
    list_filter = ['prkvigenciaid', 'tipotarifa', ]
    raw_id_fields = ('prkvigenciaid', 'classe')
    related_lookup_fields = {'fk': ['prkvigenciaid', 'classe']}
    fieldsets = [
        (_(u'Definições da tarifa'), {'fields': ['ativa', 'descricao', 'prkvigenciaid', 'classe', 'tolerancia', 'valormaximo', 'tipotarifa', 'tarifaunica', ]}),
        (_(u'Campos Escondidos'), {'fields': [('apostempo1', 'aposvalor', 'aposfracaotempo', 'aposqtdfracaotempo', 'aposrepetecobranca', 'aposqtdrepetecobrancatempo', 'aposacumulacobranca')],
                                'classes': ['hidden']})
    ]
    inlines = [tarifa_valor_inline, ]

    class Media:
        css = {'all': (settings.MEDIA_URL + 'css/jquery.modal.css', )}
        js = (
            settings.MEDIA_URL + 'js/tarifa.js',
            settings.MEDIA_URL + 'js/jquery-modal/jquery.modal.js',
            settings.MEDIA_URL + 'js/jquery-maskmoney/jquery.maskMoney.js',
            )


class ticket_form(definitions.baseform):
    # pesquisa
    list_display = ['valido', 'status_', 'dtemissao_', 'codigo', 'placa', 'dtpagamento', 'permanencia', 'valorrealizado', 'razaoinvalidade', 'Imagem_',
    'fidelidadeid', 'Imprimir_Ticket', 'imprimir_recibo_financeiro_', 'pagamento_ticket_']
    ordering = ['-dtemissao', ]
    list_filter = [('dtemissao', fields.IHXDateRangeFilter), 'valido', models.prkticket_statusfilter, 'fidelidadeid']
    search_fields = ['codigo', 'placa']
    raw_id_fields = ('classe', 'fidelidadeid')
    related_lookup_fields = {'fk': ['classe', 'fidelidadeid']}
    list_display_links = list_display
    #fieldsets = [
        #(_(u'Classe da tarifação'), {'fields': ['classe']}),
        #(_(u'Ticket'), {'fields': ['codigo', 'tolerancia', ('valido', 'razaoinvalidade')]}),
        #(_(u'Controle'), {'fields': ['dtemissao', 'dtpagamento', 'valororiginal', 'valorrealizado', 'multiploacessodia', ]}),
    #]
    date_hierarchy = 'dtemissao'
    permitir_inclusao = False
    permitir_edicao = True
    permitir_exclusao = False
    readonly_fields = ['classe', 'status_', 'codigo', 'tolerancia', 'dtemissao', 'dtpagamento', 'valororiginal', 'valorrealizado', 'valororiginal', 'placa', 'permanenciaminutos', 'permanencia',
         'pessoaid', 'fidelidadeid', 'valido', 'multiploacessodia', 'nolocal', 'tipopagamento', 'razaoinvalidade', 'veiculoimagem']

    ##############################################################################
    # RELATORIOS
    ##############################################################################
    # filtro cabecalho relatatorio:::
    def RetornaFiltroTela(self, request):
        from django.http import HttpResponseRedirect
        #Para que não seja chamado todo o momento quando carregar o form
        vSucesso = False
        filtroDict = {}
        filtroDict, vSucesso = functions.convert_request_to_filter(request, ['status_', ])
        if vSucesso is False:
            if 'HTTP_REFERER' in request.META:
                return HttpResponseRedirect(request.META['HTTP_REFERER'])
            else:
                return HttpResponseRedirect("/admin/")
        report_relatorio.criterio_filtro = ''
        report_relatorio.criterio_filtro = functions.filtro_tela_msg(filtroDict, self.model)
        #custom filter do modulo
        if 'status_' in request.GET:
            #filtro
            vstatus = request.GET.get('status_')
            if vstatus == '01':
                ##Realizado
                vstr = _(u'Realizado')
            if vstatus == '02':
                ##Não Realizado
                vstr = _(u'Não Realizado')
            #caption filter
            if report_relatorio.criterio_filtro != '':
                report_relatorio.criterio_filtro += ' | '
            report_relatorio.criterio_filtro += u'Status: %s' % (vstr)

    def Rel_Ticket(self, request, queryset):
        self.RetornaFiltroTela(request)
        return report_views.printDefault(request, reports.Rel_Ticket, self.model, queryset.order_by('dtemissao'))
    Rel_Ticket.short_description = _(u"Relatório de Tickets")

    def Rel_TicketFidelidade(self, request, queryset):
        self.RetornaFiltroTela(request)
        return report_views.printDefault(request, reports.Rel_TicketFidelidade, self.model, queryset.order_by('fidelidadeid', 'dtemissao'))
    Rel_TicketFidelidade.short_description = _(u"Relatório de Tickets - Agrupado por Fidelidade/Desconto - Detalhado")

    def Rel_TicketFidelidadeGroup(self, request, queryset):
        self.RetornaFiltroTela(request)
        return report_views.printDefault(request, reports.Rel_TicketFidelidadeGroup, self.model, queryset.order_by('fidelidadeid', 'dtemissao'))
    Rel_TicketFidelidadeGroup.short_description = _(u"Relatório de Tickets - Agrupado por Fidelidade/Desconto")

    def Rel_Grafico_PermancenciaHora(self, request, queryset):
            vSQL = '''
                SELECT  CONCAT('ATE ', ROUND(permanenciaminutos / 60) + 1, ' HORA') AS categora,
                COUNT(id) AS valor
                FROM prkticket
                WHERE
                permanenciaminutos IS NOT NULL
            '''
            #filtro interface
            filtro, vSucesso = functions.convert_request_to_filter(request)
            if filtro:
                vSQL += " AND " + functions.convert_filter_to_sql(filtro)

            vSQL += '''
                    GROUP BY classe_id, ROUND(permanenciaminutos / 60)
                    '''
            return graficos.barras(vSQL, _(u'Gráfico Permanência Hora'), _(u'Permanência Hora'), _(u'Permanência'), _(u'Hora'))
    Rel_Grafico_PermancenciaHora.short_description = _(u'Gráfico Permanência Hora')
    Rel_Grafico_PermancenciaHora.acts_on_all = True

    def Rel_Grafico_TicketMedio(self, request, queryset):
            vSQL = '''
                SELECT  CONCAT( YEAR(dtpagamento), '/', LPAD(MONTH(dtpagamento),2,'0') ) AS categoria,
                TRUNCATE(AVG(valorrealizado), 2) as valor
                FROM prkticket
                WHERE
                permanenciaminutos IS NOT NULL
            '''
            #filtro interface
            filtro, vSucesso = functions.convert_request_to_filter(request)
            if filtro:
                vSQL += " AND " + functions.convert_filter_to_sql(filtro)

            vSQL += '''
                    GROUP BY YEAR(dtpagamento), MONTH(dtpagamento)
                    '''

            return graficos.barras(vSQL, _(u'Gráfico Valor Médio Pago'), _(u'Valor Médio Agrupado por Mês'), _(u'Mês/Ano'), _(u'Valor'))
    Rel_Grafico_TicketMedio.short_description = _(u'Gráfico Valor Médio Pago - Agrupado por Mês')
    Rel_Grafico_TicketMedio.acts_on_all = True

    def Rel_Grafico_TicketTempoMedio(self, request, queryset):
            vSQL = '''
                SELECT  CONCAT( YEAR(dtpagamento), '/', LPAD(MONTH(dtpagamento),2,'0') ) AS categoria,
                TRUNCATE(AVG(ROUND(permanenciaminutos / 60)), 2) AS Valor
                FROM prkticket
                WHERE
                permanenciaminutos IS NOT NULL
            '''
            #filtro interface
            filtro, vSucesso = functions.convert_request_to_filter(request)
            if filtro:
                vSQL += " AND " + functions.convert_filter_to_sql(filtro)

            vSQL += '''
                    GROUP BY YEAR(dtpagamento), MONTH(dtpagamento)
                    '''

            return graficos.barras(vSQL, _(u'Gráfico Tempo Médio Permanência'), _(u'Tempo Médio Agrupado por Mês'), _(u'Mês/Ano'), _(u'Hora'))
    Rel_Grafico_TicketTempoMedio.short_description = _(u'Gráfico Tempo Médio - Agrupado por Mês')
    Rel_Grafico_TicketTempoMedio.acts_on_all = True

    def Baixar_TicketAtivo(self, request, queryset):
        queryset.update(valido=False, razaoinvalidade="Baixa manual de ticket")
        self.message_user(request, "Baixa de tickets realizada com sucesso!")
    Baixar_TicketAtivo.short_description = _(u"Baixar Ticket - Realizar baixa manual de tickets")

    def gerar_permanencia(self, request, queryset):
        queryset = queryset.filter(dtpagamento__isnull=False)
        for item in queryset:
            if item.dtpagamento:
                item.permanencia = item.dtpagamento - item.dtemissao

                #days, seconds = item.permanencia.days, item.permanencia.seconds
                #hours = days * 24 + seconds // 3600
                #minutes = (seconds % 3600) // 60
                #seconds = seconds % 60

                item.permanenciaminutos = int(item.permanencia.total_seconds()) / 60
                item.save()

        self.message_user(request, "Permanencia gerada: " + str(queryset.count()))
    gerar_permanencia.short_description = _(u"Gerar permanencia tickets")

    #acoes do modulo
    actions = [Rel_Ticket, gerar_permanencia]


class ticket_desconto_form(ticket_form):
    # pesquisa
    list_display = ['valido', 'dtemissao', 'codigo', 'dtpagamento', 'valorrealizado', 'razaoinvalidade', ]
    search_fields = ['codigo', ]
    raw_id_fields = ('fidelidadeid', )
    list_filter = ('fidelidadeid', )
    related_lookup_fields = {'fk': ['fidelidadeid', ]}
    readonly_fields = ['codigo']
    list_display_links = list_display
    fieldsets = [
        (_(u'Ticket'), {'fields': ['codigo', 'fidelidadeid', ]}),
    ]
    permitir_inclusao = False
    permitir_edicao = True

    def queryset(self, request):
        qs = super(ticket_desconto_form, self).queryset(request)
        return qs.filter(dtpagamento=None)


class prktarifaclasse_form(definitions.baseform):
    # pesquisa
    list_display = ['descricao', ]
    search_fields = ['descricao', ]
    list_display_links = list_display


class prkticket_mensagem_form(definitions.baseform):
    permitir_inclusao = False
    permitir_exclusao = False
    # pesquisa
    list_display = ['linha1', ]
    list_display_links = list_display


class ticket_finalizado_form(ticket_form):
    list_display = ['dtpagamento', 'dtemissao_', 'codigo', 'placa', 'Imagem_', 'permanencia', 'valorrealizado', 'fidelidadeid', 'Imprimir_Ticket', 'imprimir_recibo_financeiro_']
    ordering = ['-dtpagamento', ]
    list_filter = [('dtpagamento', fields.IHXDateRangeFilter), 'fidelidadeid']
    search_fields = ['codigo', 'placa']
    list_display_links = list_display
    date_hierarchy = 'dtpagamento'
    permitir_inclusao = False
    #permitir_edicao = False
    readonly_fields = ['classe', 'status_', 'codigo', 'tolerancia', 'dtemissao', 'dtpagamento', 'valororiginal', 'valorrealizado', 'valororiginal', 'placa', 'permanenciaminutos', 'permanencia',
         'pessoaid', 'fidelidadeid', 'valido', 'multiploacessodia', 'nolocal', 'tipopagamento', 'razaoinvalidade', 'veiculoimagem']

    def queryset(self, request):
        qs = super(ticket_finalizado_form, self).queryset(request)
        return qs.filter(Q(dtpagamento__isnull=False))

    #acoes do modulo aqui adiciono todos os demais
    actions = [ticket_form.Rel_TicketFidelidadeGroup, ticket_form.Rel_TicketFidelidade, ticket_form.Rel_Grafico_PermancenciaHora,
    ticket_form.Rel_Grafico_TicketMedio, ticket_form.Rel_Grafico_TicketTempoMedio]


class prkticket_ativo_form(ticket_form):
    list_display = ['valido', 'status_', 'dtemissao_', 'codigo', 'placa', 'Imagem_', 'dtpagamento', 'permanencia', 'valorrealizado', 'razaoinvalidade', 'fidelidadeid', 'pagamento_ticket_']
    ordering = ['-dtemissao', ]
    list_filter = [('dtemissao', fields.IHXDateRangeFilter), models.prkticket_statusfilter, 'fidelidadeid']
    search_fields = ['codigo', 'placa']
    list_display_links = []
    date_hierarchy = 'dtemissao'
    permitir_inclusao = False
    #permitir_edicao = False

    def queryset(self, request):
        qs = super(prkticket_ativo_form, self).queryset(request)
        return qs.filter(Q(valido=True, dtpagamento__isnull=True))

    #acoes do modulo aqui adiciono todos os demais
    actions = [ticket_form.Baixar_TicketAtivo]