# -*- coding: UTF-8 -*-
from datetime import datetime
from django.utils.translation import ugettext as _
from django.db import models
from django.db import transaction
from django.core.exceptions import ValidationError
from django.contrib.admin import SimpleListFilter
from ihx.access.core.MODPessoa.models import pessoa
from ihx.access.base import fields, definitions, functions, constants, modulos
from ihx.access.core.MODControle.models import planofidelidade
from ihx.access.core.MODEmpresa.models import empresa
from app import settings
from app.modules import app_constants


class prkvigencia(definitions.abstract_ihxbasemodel):
    tempo1 = models.TimeField('De')
    tempo2 = models.TimeField('Até')
    seg = fields.TECBooleanField(_(u'Seg'), default=True)
    ter = fields.TECBooleanField(_(u'Ter'), default=True)
    qua = fields.TECBooleanField(_(u'Qua'), default=True)
    qui = fields.TECBooleanField(_(u'Qui'), default=True)
    sex = fields.TECBooleanField(_(u'Sex'), default=True)
    sab = fields.TECBooleanField(_(u'Sáb'), default=True)
    dom = fields.TECBooleanField(_(u'Dom'), default=True)

    def __unicode__(self):
        return u'De %s - até %s - %s' % (self.tempo1, self.tempo2, self.dias_())

    def seg_(self):
        if self.seg:
            return _(u'Seg ')
        return _(u' ')

    def ter_(self):
        if self.ter:
            return _(u'Ter ')
        return _(u' ')

    def qua_(self):
        if self.qua:
            return _(u'Qua ')
        return _(u' ')

    def qui_(self):
        if self.qui:
            return _(u'Qui ')
        return _(u' ')

    def sex_(self):
        if self.sex:
            return _(u'Sex ')
        return _(u' ')

    def sab_(self):
        if self.sab:
            return _(u'Sab ')
        return _(u' ')

    def dom_(self):
        if self.dom:
            return _(u'Dom ')
        return _(u' ')

    def dias_(self):
        if self.seg and self.ter and self.qua and self.qui and self.sex and self.sab and self.dom:
            return _(u'Todos os dias')
        return _(u'Dias: ') + self.seg_() + self.ter_() + self.qua_() + self.qui_() + self.sex_() + self.sab_() + self.dom_()

    def save(self, *args, **kwargs):
        from datetime import time
        if self.tempo2:
            if self.tempo2.second == 0:
                self.tempo2 = time(self.tempo2.hour, self.tempo2.minute, 59)
        super(prkvigencia, self).save(*args, **kwargs)

    class Meta:
        db_table = 'prkvigencia'
        verbose_name = _(u'Vigência')
        verbose_name_plural = _(u'Vigências')
        app_label = 'MODManutencao'


class prktarifaclasse(definitions.abstract_ihxbasemodel):
    descricao = models.CharField(_(u'Descricão'), max_length=50, unique=True)

    def __unicode__(self):
        return u'%s' % (self.descricao)

    class Meta:
        db_table = 'prktipotarifacao'
        verbose_name = _(u'Classificação da tarifa')
        verbose_name_plural = _(u'Classificação das tarifas')
        #app_label = 'MODManutencao'


class prktarifa(definitions.abstract_ihxbasemodel):
    classe = models.ForeignKey(prktarifaclasse, verbose_name=_(u'Classificação da tarifa'), null=True, blank=True)
    dtcadastro = models.DateTimeField(_(u'Data do cadastro'), auto_now_add=True)
    prkvigenciaid = models.ForeignKey(prkvigencia, verbose_name=_(u'Vigência'))
    descricao = models.CharField(_(u'Descricão'), max_length=50, unique=True)
    ativa = models.BooleanField(_(u'Tarifa ativa'), default=True)
    tolerancia = models.TimeField(_(u'Tolerância'), help_text='Define a tolerância de saída após o pagamento.', null=True, blank=True)
    valormaximo = fields.CurrencyField(_(u'Valor máximo'), help_text='Define o valor máximo de cobrança por dia.', null=True, blank=True)
    tipotarifa = models.SmallIntegerField(_(u'Tipo de tarifa'), choices=app_constants.TIPOTARIFA, default=1, )
    tarifaunica = fields.CurrencyField(_(u'Tarifa única'), null=True, blank=True)
    apostempo1 = models.TimeField(null=True, blank=True)
    aposvalor = fields.CurrencyField(_(u'Valor'), null=True, blank=True)
    aposfracaotempo = fields.IHXBooleanField(_(u'Permitir fração'), null=True)
    aposqtdfracaotempo = models.SmallIntegerField(_(u'Fração partes'), null=True, blank=True, max_length=2)
    aposrepetecobranca = fields.IHXBooleanField(_(u'Repetir cobrança'), null=True)
    aposqtdrepetecobrancatempo = models.SmallIntegerField(_(u'A cada minutos'), null=True, blank=True, max_length=2)
    aposacumulacobranca = fields.IHXBooleanField(_(u'Acumula cobrança'), null=True)

    def __unicode__(self):
        return u'%s' % (self.descricao)

    def clean_fields(self, exclude=None):
        super(prktarifa, self).clean_fields(exclude)
        errors = {}
        if self.tipotarifa == 2 and self.tarifaunica is None:
            errors['tarifaunica'] = functions.preparar_msg_erro(_(u'Foi selecionado "Preço Fixo", por favor informe o valor da tarifa única'))

        #if not self.valormaximo and not self.aposvalor and self.tipotarifa == 1:
            #errors['valormaximo'] = functions.preparar_msg_erro(_(u'Preencha o campo "Valor máximo" ou "valor maximo cobrado após"'))

        if errors:
            raise ValidationError(errors)

    def save(self, *args, **kwargs):
        if self.tipotarifa == 2:
            self.apostempo1 = None
            self.aposvalor = None
        super(prktarifa, self).save(*args, **kwargs)

    class Meta:
        db_table = 'prktarifa'
        verbose_name = _(u'Tarifa')
        verbose_name_plural = _(u'Tarifas')
        app_label = 'MODManutencao'


class prkvalor(definitions.abstract_ihxbasemodel):
    tarifaid = models.ForeignKey(prktarifa, verbose_name=_(u'Tarifa'))
    acumulacobranca = fields.IHXBooleanField(_(u'Acumula cobrança'), null=True)
    tempo1 = models.TimeField(u'De', null=True, blank=True)
    tempo2 = models.TimeField(u'Até', null=True, blank=True)
    valor = fields.CurrencyField(_(u'Valor'), )
    fracaotempo = fields.IHXBooleanField(_(u'Permitir fração'), null=True)
    qtdfracaotempo = models.SmallIntegerField(_(u'Fração partes'), null=True, blank=True, max_length=2)
    repetecobranca = fields.IHXBooleanField(_(u'Repetir cobrança'), null=True)
    qtdrepetecobrancatempo = models.SmallIntegerField(_(u'A cada minutos'), null=True, blank=True, max_length=2)

    class Meta:
        db_table = 'prkvalor'
        verbose_name = _(u'Valor')
        verbose_name_plural = _(u'Valores')

    def save(self, *args, **kwargs):
        from datetime import time
        if self.tempo2:
            if self.tempo2.second == 0:
                self.tempo2 = time(self.tempo2.hour, self.tempo2.minute, 59)
        super(prkvalor, self).save(*args, **kwargs)

    def clean_fields(self, exclude=None):
        super(prkvalor, self).clean_fields(exclude)
        errors = {}
        if not (self.tempo1 and self.tempo2):
            errors['tempo1'] = functions.preparar_msg_erro(_(u'Por favor informe o tempo incial e final'))
            errors['tempo2'] = functions.preparar_msg_erro(_(u'Por favor informe o tempo incial e final'))
        #valida período
        if (self.tempo1 and self.tempo2):
            if self.tempo1 >= self.tempo2:
                errors['tempo1'] = functions.preparar_msg_erro(_(u'Horário não é válido! O "Horário final" deverá ser maior que o "Horário inicial"'))
                errors['tempo2'] = functions.preparar_msg_erro(_(u'Horário não é válido! O "Horário final" deverá ser maior que o "Horário inicial"'))

        if (self.fracaotempo and self.qtdfracaotempo < 0):
            errors['qtdfracaotempo'] = functions.preparar_msg_erro(_(u'Favor informe o valor das partes a serem fracionadas'))

        if errors:
            raise ValidationError(errors)


class abstract_prkticket(definitions.abstract_ihxbasemodel):
    codigo = models.CharField(_(u'Código'), unique=True, max_length=32, null=False, blank=False)
    classe = models.ForeignKey(prktarifaclasse, related_name='%(class)s_classe_original', verbose_name=_(u'Classe da tarifa'), null=True, blank=True)
    dtemissao = models.DateTimeField(_(u'Emissão'), )
    dtpagamento = models.DateTimeField(_(u'Pagamento'), null=True, blank=True)
    tolerancia = models.DateTimeField(_(u'Tolerância'), null=True, blank=True)
    valororiginal = fields.CurrencyField(_(u'Valor original'), null=True, blank=True)
    valorrealizado = fields.CurrencyField(_(u'Valor realizado'), null=True, blank=True)
    valido = models.BooleanField(_(u'No Pátio'), default=False)
    razaoinvalidade = models.TextField(_(u'Descrição'), null=True, blank=True, help_text=_(u'Descreve as razões de não validade do ticket.'))
    multiploacessodia = models.BooleanField(_(u'Multiplos acessos'), default=False, help_text=_(u'Define que o ticket poderá realizar múltiplos acessos durante o dia.'))
    placa = models.CharField(_(u'Placa'), max_length=32, null=True, blank=True)
    fidelidadeid = models.ForeignKey(planofidelidade, verbose_name=_(u'Desconto'), null=True, blank=True)
    nolocal = fields.IHXBooleanField(_(u'No local'), default=False)
    tipopagamento = models.SmallIntegerField(_(u'Forma de pagamento'), choices=constants.FINFORMAPAGAMENTO, default=1, null=True, blank=True)
    pessoaid = models.ForeignKey(pessoa, verbose_name=_(u'Pessoa'), null=True, blank=True)
    permanenciaminutos = models.IntegerField(_(u'Minutos'), null=True, blank=True)
    permanencia = models.CharField(_(u'Permanência'), max_length=50, null=True, blank=True)
    veiculoimagem = models.ImageField(_(u'Imagem'), upload_to='photos/prkticket', null=True, blank=True)

    def dtemissao_(self):
        if self.dtemissao is None:
            return '--/--/----  00:00'
        else:
            return self.dtemissao.strftime("%d/%m/%Y %H:%M")
    dtemissao_.short_description = u'Emissão'
    dtemissao_.admin_order_field = 'dtemissao'

    def status_(self):
        if self.dtpagamento is None:
            return '''<div style="text-align: center;">
                    <img src="%s/admin/img/status-cinza.png" title="Não realizado" alt="Não realizado" height="16" />
              </div>''' % (settings.TEMPLATE_URL)
        else:
            return '''<div style="text-align: center;">
                            <img src="%s/admin/img/status-verde.png" title="Realizado" alt="Realizado" height="16" />
                     </div>''' % (settings.TEMPLATE_URL)
    status_.allow_tags = True

    def Imagem_(self, max_size='50'):
        return functions.get_imagem_modal(self.veiculoimagem, max_size, 'height', self.id)
    Imagem_.allow_tags = True

    def imprimir_recibo_financeiro_(self):
        from ihx.access.core.MODFinanceiro.models import finmovimento
        if finmovimento.objects.filter(prkticketid=self.pk).exists():
            vFinmovimento = finmovimento.objects.filter(prkticketid=self.pk)[0]
            if vFinmovimento.dtpagamento:
                return  '''<div style="text-align: center;"><a target="_blank" href="/imprimir_recibo/%s">
                    <img src="%s/admin/img/imprimir.png" title="Imprimir Ticket" alt="ImprimirTicket" height="16">
                </a></div>''' % (str(vFinmovimento.pk), settings.TEMPLATE_URL)
        else:
            return ''
    imprimir_recibo_financeiro_.short_description = _(u"Imprimir Recibo")
    imprimir_recibo_financeiro_.allow_tags = True

    def pagamento_ticket_(self):
        vCodigoTicket = prkticket.objects.filter(codigo=self.codigo)[0]
        if vCodigoTicket.dtpagamento is None:
            if vCodigoTicket:
                return  '''<div style="text-align: center;"><a target="_blank" href="/pagamento/%s">
                    <img src="%s/admin/img/ticket.png" title="Imprimir Ticket" alt="ImprimirTicket" height="16">
                </a></div>''' % (str(vCodigoTicket), settings.TEMPLATE_URL)
        else:
            return ''
    pagamento_ticket_.short_description = _(u"Pagar Ticket")
    pagamento_ticket_.allow_tags = True

    class Meta:
        managed = False
        abstract = True
        ordering = ['-dtemissao']


class prkticket_statusfilter(SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _(u'Status')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'status_'

    def lookups(self, request, model_admin):
        return (
            ('01', _(u'Realizado')),
            ('02', _(u'Não realizado')),
        )

    def queryset(self, request, queryset):
        if self.value() == '01':
            #Realizado
            return queryset.filter(dtpagamento__isnull=False)
        if self.value() == '02':
            #Não Realizado
            return queryset.filter(dtpagamento__isnull=True)


class prkticket(abstract_prkticket):
    def Imprimir_Ticket(self):
        #import os
        #vArquivo = settings.RAIZ + settings.MEDIA_URL + "/pdf/%s.pdf" % str(self.codigo)
        #if os.path.isfile(vArquivo):
                #Modificar para imprimir_ticket2 na url, para gerar o PDF ao inves de abrir no servidor
        return  '''<div style="text-align: center;"><a target="_blank" href="/ticket_impresso/%s">
                    <img src="%s/admin/img/imprimir.png" title="Imprimir Ticket" alt="ImprimirTicket" height="16">
                </a></div>''' % (str(self.codigo), settings.TEMPLATE_URL)
        #else:
            #return ''
    Imprimir_Ticket.short_description = _(u"Imprimir Ticket")
    Imprimir_Ticket.allow_tags = True

    def __unicode__(self):
        return u'%s' % (self.codigo)

    class Meta:
        db_table = 'prkticket'
        abstract = False
        verbose_name = _(u'Ticket')


class prkticket_am(abstract_prkticket):

    class Meta:
        db_table = 'prkticket_am'
        abstract = False
        verbose_name = _(u'Ticket arquivo morto')


class prkticket_desconto(prkticket):
    class Meta:
        db_table = 'prkticket'
        proxy = True
        verbose_name = _(u'Conceder desconto')
        verbose_name_plural = _(u'Conceder desconto')
        app_label = 'MODManutencao'


class prkticket_mensagem(definitions.abstract_ihxbasemodel):
    linha1 = models.CharField(_(u'Linha 1'), max_length=38, null=True, blank=True)
    linha2 = models.CharField(_(u'Linha 2'), max_length=38, null=True, blank=True)
    linha3 = models.CharField(_(u'Linha 3'), max_length=38, null=True, blank=True)
    linha4 = models.CharField(_(u'Linha 4'), max_length=38, null=True, blank=True)

    class Meta:
        db_table = 'prkticket_mensagem'
        verbose_name = _(u'Configurações mensagem ticket')
        verbose_name_plural = _(u'Configurações mensagens tickets')
        app_label = 'MODManutencao'

    def __unicode__(self):
        return u'Mensagens configuradas'


class prkticket_finalizado(prkticket):
    class Meta:
        db_table = 'prkticket'
        proxy = True
        verbose_name = _(u'Ticket Finalizado')
        verbose_name_plural = _(u'Tickets Finalizados')


class prkticket_ativo(prkticket):
    class Meta:
        db_table = 'prkticket'
        proxy = True
        verbose_name = _(u'Ticket no pátio')
        verbose_name_plural = _(u'Tickets no pátio')