# -*- coding: utf-8 -*-
from geraldo import ReportBand, ObjectValue, BAND_WIDTH, Label, ReportGroup
from reportlab.lib.units import cm
from reportlab.lib.enums import TA_RIGHT
from ihx.access.core.MODRelatorio.reports import TECreportbaseDefault


class Rel_Ticket(TECreportbaseDefault):
    title = 'Relatório de Tickets'

    class band_begin(ReportBand):

        height = 0.5 * cm
        elements = [
                    Label(text=u"Emissão", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=0 * cm),
                    Label(text=u"Pagamento", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=3.5 * cm),
                    Label(text=u"Código", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=7 * cm),
                    Label(text=u"Permanência", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=10.5 * cm),
                    Label(text=u"Valor", style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}, top=0 * cm, left=14.9 * cm),
                   ]
        borders = {'bottom': True}

    class band_detail(ReportBand):
        auto_expand_height = True
        height = 0.5 * cm
        elements = (
                ObjectValue(attribute_name='dtemissao',
                    left=0 * cm),
                ObjectValue(attribute_name='dtpagamento',
                    get_value=lambda instance: instance.dtpagamento.strftime('%d/%m/%Y %H:%M:%S') if instance.dtpagamento else '--/--/----',
                    left=3.5 * cm),
                ObjectValue(attribute_name='codigo', left=7 * cm),
                ObjectValue(attribute_name='permanencia',
                    get_value=lambda instance: instance.permanencia if instance.permanencia else '',
                    left=10.5 * cm),
                ObjectValue(attribute_name='valorrealizado',
                    get_value=lambda instance: instance.valorrealizado if instance.valorrealizado else 0,
                    left=9.9 * cm, width=10 * cm, style={'alignment': TA_RIGHT}),
                )
    groups = [
            ReportGroup(
                attribute_name='classe',
                band_header=ReportBand(
                height=0.7 * cm,
                elements=[ObjectValue(attribute_name='classe',
                        left=0, top=0.1 * cm, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 10, 'backColor': '#cfcfcf'}),
                          ],
                          ),
                band_footer=ReportBand(
                    height=0.9 * cm,
                    elements=[
                              Label(text='Valor Total:', top=0 * cm, style={'fontName': 'Helvetica-Bold', }),
                              ObjectValue(expression='sum(valorrealizado)', top=0 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                              Label(text='Total tickets:', top=0.4 * cm, style={'fontName': 'Helvetica-Bold', }),
                              ObjectValue(expression='count(dtemissao)', top=0.4 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                             ],
                    borders={'top': True, 'bottom': True},
                            ),
                          ),
            ]


class Rel_TicketFidelidade(TECreportbaseDefault):
    title = 'Relatório de Tickets  - Agrupado por Fidelidade/Desconto - Detalhado'

    class band_begin(ReportBand):

        height = 0.5 * cm
        elements = [
                    Label(text=u"Emissão", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=0 * cm),
                    Label(text=u"Pagamento", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=3.5 * cm),
                    Label(text=u"Código", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=7 * cm),
                    Label(text=u"Permanência", style={'fontName': 'Helvetica-Bold'}, top=0 * cm, left=10.5 * cm),
                    Label(text=u"Valor", style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}, top=0 * cm, left=14.9 * cm),
                   ]
        borders = {'bottom': True}

    class band_detail(ReportBand):
        auto_expand_height = True
        height = 0.5 * cm
        elements = (
                ObjectValue(attribute_name='dtemissao', left=0 * cm),
                ObjectValue(attribute_name='dtpagamento',
                    get_value=lambda instance: instance.dtpagamento.strftime('%d/%m/%Y %H:%M:%S') if instance.dtpagamento else '--/--/----',
                    left=3.5 * cm),
                ObjectValue(attribute_name='codigo', left=7 * cm),
                ObjectValue(attribute_name='permanencia',
                    get_value=lambda instance: instance.permanencia if instance.permanencia else '',
                    left=10.5 * cm),
                ObjectValue(attribute_name='valorrealizado',
                    get_value=lambda instance: instance.valorrealizado if instance.valorrealizado else 0,
                    left=9.9 * cm, width=10 * cm, style={'alignment': TA_RIGHT}),
                )
    groups = [
            ReportGroup(
                attribute_name='fidelidadeid',
                band_header=ReportBand(
                height=0.7 * cm,
                elements=[ObjectValue(attribute_name='fidelidadeid',
                        get_value=lambda instance: instance.fidelidadeid if instance.fidelidadeid else 'Tarifado',
                        left=0, top=0.1 * cm, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 10, 'backColor': '#cfcfcf'}),
                          ],
                          ),
                band_footer=ReportBand(
                    height=0.9 * cm,
                    elements=[
                              Label(text='Valor Total:', top=0 * cm, style={'fontName': 'Helvetica-Bold', }),
                              ObjectValue(expression='sum(valorrealizado)', top=0 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                              Label(text='Total tickets:', top=0.4 * cm, style={'fontName': 'Helvetica-Bold', }),
                              ObjectValue(expression='count(dtemissao)', top=0.4 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                             ],
                    borders={'top': True, 'bottom': True},
                            ),
                          ),
            ]


class Rel_TicketFidelidadeGroup(TECreportbaseDefault):
    title = 'Relatório de Tickets  - Agrupado por Fidelidade/Desconto'

    class band_begin(ReportBand):

        height = 0.5 * cm
        elements = [
                   ]
        borders = {'bottom': True}

    class band_detail(ReportBand):
        auto_expand_height = True
        height = 0.5 * cm
        elements = (
                )
    groups = [
            ReportGroup(
                attribute_name='fidelidadeid',
                band_header=ReportBand(
                height=0.7 * cm,
                elements=[ObjectValue(attribute_name='fidelidadeid',
                        get_value=lambda instance: instance.fidelidadeid if instance.fidelidadeid else 'Tarifado',
                        left=0, top=0.1 * cm, width=BAND_WIDTH,
                        style={'fontName': 'Helvetica-Bold', 'fontSize': 10, 'backColor': '#cfcfcf'}),
                          ],
                          ),
                band_footer=ReportBand(
                    height=0.9 * cm,
                    elements=[
                              Label(text='Valor total:', top=0 * cm, style={'fontName': 'Helvetica-Bold', }),
                              ObjectValue(expression='sum(valorrealizado)', top=0 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                              Label(text='Total tickets:', top=0.4 * cm, style={'fontName': 'Helvetica-Bold', }),
                              ObjectValue(expression='count(dtemissao)', top=0.4 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                             ],
                    #borders={'top': True, 'bottom': True},
                            ),
                          ),
            ]

    class band_summary(ReportBand):
        auto_expand_height = True
        elements = [
                  Label(text='Valor total:', top=0 * cm, style={'fontName': 'Helvetica-Bold', }),
                  ObjectValue(expression='sum(valorrealizado)', top=0 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
                  Label(text='Total tickets:', top=0.4 * cm, style={'fontName': 'Helvetica-Bold', }),
                  ObjectValue(expression='count(dtemissao)', top=0.4 * cm, left=14.9 * cm, style={'fontName': 'Helvetica-Bold', 'alignment': TA_RIGHT}),
        ]
        borders = {'top': True}